@@ -1,20 +1,26 @@
+import { existsSync } from 'fs';
+import path, { isAbsolute } from 'path';
 import * as vscode from 'vscode';
 import {
     Disposable,
     ExecuteCommandParams,
     ExecuteCommandRequest,
-    LanguageClient,
 } from 'vscode-languageclient/node';
 import { AdaCodeLensProvider } from './AdaCodeLensProvider';
-import { createClient } from './clients';
+import { AdaLanguageClient, createClient } from './clients';
+import {
+    CMD_RELOAD_PROJECT,
+    CMD_RESTART_LANG_SERVERS,
+    CMD_SHOW_ADA_LS_OUTPUT,
+    CMD_SHOW_EXTENSION_LOGS,
+    CMD_SHOW_GPR_LS_OUTPUT,
+} from './commands';
 import { AdaInitialDebugConfigProvider, initializeDebugging } from './debugConfigProvider';
 import { logger } from './extension';
-import { existsSync } from 'fs';
-import path from 'path';
 import { GnatTaskProvider } from './gnatTaskProvider';
 import { initializeTesting } from './gnattest';
 import { GprTaskProvider } from './gprTaskProvider';
-import { getArgValue, TERMINAL_ENV_SETTING_NAME, exe, getEvaluatedTerminalEnv } from './helpers';
+import { TERMINAL_ENV_SETTING_NAME, exe, getArgValue, getEvaluatedTerminalEnv } from './helpers';
 import {
     SimpleTaskDef,
     SimpleTaskProvider,
@@ -23,7 +29,6 @@ import {
     createAdaTaskProvider,
     createSparkTaskProvider,
 } from './taskProviders';
-import { isAbsolute } from 'path';
 
 /**
  * Return type of the 'als-source-dirs' LSP request.
@@ -43,8 +48,8 @@ export type ALSSourceDirDescription = {
  * wherever needed.
  */
 export class ExtensionState {
-    public readonly adaClient: LanguageClient;
-    public readonly gprClient: LanguageClient;
+    public readonly adaClient: AdaLanguageClient;
+    public readonly gprClient: AdaLanguageClient;
     public readonly context: vscode.ExtensionContext;
     public readonly dynamicDebugConfigProvider: {
         provideDebugConfigurations(
@@ -58,6 +63,7 @@ export class ExtensionState {
     public readonly codelensProvider = new AdaCodeLensProvider();
     public readonly testController: vscode.TestController;
     public readonly testData: Map<vscode.TestItem, object> = new Map();
+    public readonly statusBar: vscode.StatusBarItem;
 
     /**
      * The following fields are caches for ALS requests or costly properties.
@@ -71,6 +77,7 @@ export class ExtensionState {
     cachedAlireTomls: vscode.Uri[] | undefined;
     cachedDebugServerAddress: string | undefined | null;
     cachedGdb: string | undefined | null = undefined;
+    projectAttributeCache: Map<string, Promise<string | string[]>> = new Map();
 
     private adaTaskProvider?: SimpleTaskProvider;
     private sparkTaskProvider?: SimpleTaskProvider;
@@ -85,6 +92,7 @@ export class ExtensionState {
         this.cachedAlireTomls = undefined;
         this.cachedDebugServerAddress = undefined;
         this.cachedGdb = undefined;
+        this.projectAttributeCache.clear();
     }
 
     constructor(context: vscode.ExtensionContext) {
@@ -108,6 +116,8 @@ export class ExtensionState {
         this.initialDebugConfigProvider = result.providerInitial;
         this.dynamicDebugConfigProvider = result.providerDynamic;
         this.testController = initializeTesting(context);
+        this.statusBar = vscode.window.createStatusBarItem(vscode.StatusBarAlignment.Left);
+        this.context.subscriptions.push(this.statusBar);
     }
 
     public start = async () => {
@@ -116,6 +126,7 @@ export class ExtensionState {
         this.context.subscriptions.push(
             vscode.languages.registerCodeLensProvider('ada', this.codelensProvider),
         );
+        this.updateStatusBarVisibility(undefined);
     };
 
     public dispose = () => {
@@ -148,6 +159,139 @@ export class ExtensionState {
         ];
     };
 
+    /**
+     * Update the status bar item's visibility according to the current context.
+     */
+    public updateStatusBarVisibility = (editor: vscode.TextEditor | undefined) => {
+        const activeEditor = editor ?? vscode.window.activeTextEditor;
+
+        // Show the status bar if the active editor is on an Ada or a GPR source, or
+        // if it's the Output view that is focused (i.e: when the active editor's
+        // document scheme is set to 'output') and showing Ada & SPARK extension's logs.
+        if (
+            activeEditor &&
+            ((activeEditor.document.uri.scheme == 'output' &&
+                activeEditor.document.fileName.includes('AdaCore')) ||
+                ['ada', 'gpr'].includes(activeEditor.document.languageId))
+        ) {
+            this.statusBar.show();
+        } else {
+            this.statusBar.hide();
+        }
+    };
+
+    /**
+     * Update the status bar item's content according to currently displayed
+     * diagnostics.
+     */
+    public updateStatusBarItem = () => {
+        // Use markdown for the status bar item tiooltip. This allows to have
+        // hyperlinks that run actual commands.
+        this.statusBar.tooltip = new vscode.MarkdownString('', true);
+        this.statusBar.tooltip.isTrusted = true;
+
+        // Show the Problems view by default when clicking on the status
+        // bar item.
+        this.statusBar.command = 'workbench.panel.markers.view.focus';
+        let alireProjectLoaded = false;
+
+        // Monitor diagnostics related to project-loading in general, including
+        // potential issues reported by Alire.
+        const PROJECT_DIAGS_SOURCE = 'ada.project';
+        const ALIRE_DIAGS_SOURCE = 'ada.alire';
+        const diagnosticSources = [PROJECT_DIAGS_SOURCE, ALIRE_DIAGS_SOURCE];
+
+        // Gather all the diagnostics from the interesting ALS diagnostics' sources.
+        // For the status bar we are interested only in project-related diagnostics.
+        const alsDiagnostics: vscode.Diagnostic[] = vscode.languages
+            .getDiagnostics()
+            .flatMap(([, diagnostics]) => diagnostics)
+            .filter((diag) => diagnosticSources.includes(diag.source ?? ''));
+
+        // Update the status bar according to the ALS project-related diagnostics
+        if (alsDiagnostics.length > 0) {
+            // Get the highest severity of the currently displayed project-diagnostics, to
+            // update the status bar's colors accordingly.
+            const statusBarSeverity: vscode.DiagnosticSeverity = alsDiagnostics
+                .map((a) => a.severity)
+                .reduce((a, b) => (a < b ? a : b));
+            this.statusBar.text = 'Ada & SPARK';
+
+            switch (statusBarSeverity) {
+                case vscode.DiagnosticSeverity.Error:
+                    this.statusBar.tooltip.appendMarkdown(
+                        'Project loading has issued errors, see the [Problems]' +
+                            '(command:workbench.panel.markers.view.focus) view' +
+                            ' for more information.',
+                    );
+                    this.statusBar.backgroundColor = new vscode.ThemeColor(
+                        'statusBarItem.errorBackground',
+                    );
+                    this.statusBar.color = new vscode.ThemeColor('statusBarItem.errorForeground');
+                    break;
+
+                case vscode.DiagnosticSeverity.Warning:
+                    this.statusBar.tooltip.appendMarkdown(
+                        'Project loading has issued warnings, see the [Problems]' +
+                            '(command:workbench.panel.markers.view.focus) view' +
+                            ' for more information.',
+                    );
+                    this.statusBar.backgroundColor = new vscode.ThemeColor(
+                        'statusBarItem.warningBackground',
+                    );
+                    this.statusBar.color = new vscode.ThemeColor('statusBarItem.warningForeground');
+                    break;
+
+                default:
+                    this.statusBar.backgroundColor = undefined;
+                    this.statusBar.color = undefined;
+
+                    // Check if we have successfully loaded the project through Alire
+                    // and adapt the status bar item's text contents if it's the case, to
+                    // mention that Alire was used for project-loading.
+                    alireProjectLoaded = alsDiagnostics.some(
+                        (diagnostic) =>
+                            diagnostic.source == PROJECT_DIAGS_SOURCE &&
+                            diagnostic.message.includes('Alire'),
+                    );
+
+                    if (alireProjectLoaded) {
+                        this.statusBar.text += ' (Alire)';
+                        this.statusBar.tooltip.appendMarkdown(
+                            'Project was loaded successfully through Alire',
+                        );
+                    } else {
+                        this.statusBar.tooltip.appendMarkdown('Project was loaded successfully.');
+                    }
+            }
+        } else {
+            // We don't have any project-related diagnostics, just clear any color/background color
+            // of the status bar.
+            this.statusBar.tooltip.appendMarkdown('Project was loaded successfully.');
+            this.statusBar.backgroundColor = undefined;
+            this.statusBar.color = undefined;
+        }
+
+        if (this.statusBar.tooltip.value) {
+            this.statusBar.tooltip.appendMarkdown('\n\n---\n\n');
+        }
+        this.statusBar.tooltip.appendMarkdown(
+            `[$(terminal) Open Extension Logs](command:${CMD_SHOW_EXTENSION_LOGS}
+ "Show Ada Extension Output")
+
+[$(terminal) Open Logs for Ada & SPARK](command:${CMD_SHOW_ADA_LS_OUTPUT}
+ "Show Ada Language Server for Ada & SPARK Output")
+
+[$(terminal) Open Logs for GPR](command:${CMD_SHOW_GPR_LS_OUTPUT}
+ "Show Ada Language Server for GPR Output")
+
+[$(refresh) Reload Project](command:${CMD_RELOAD_PROJECT} "Reload Project")
+
+[$(debug-restart) Restart Language Servers](command:${CMD_RESTART_LANG_SERVERS}
+ "Restart Ada Language Servers")`,
+        );
+    };
+
     /**
      * Unregister all the task disposables needed by the extension (e.g: task providers,
      * listeners...).
@@ -224,35 +368,63 @@ export class ExtensionState {
      * an exception when the queried attribute is not known
      * (i.e: not registered in GPR2's knowledge database).
      *
+     * Query results are cached by default so that querying the same attribute
+     * multiple times will result in only one request to the server.
+     *
      * @param attribute - The name of the project attribute (e.g: 'Target')
      * @param pkg - The name of the attribute's package (e.g: 'Compiler').
      * Can be empty for project-level attributes.
      * @param index - Attribute's index, if any. Can be a file or a language
      * (e.g: 'for Runtime ("Ada") use...').
+     * @param useCache - whether to use cached result from previous query. If
+     * set to false, the request will be sent to the server and the result will
+     * be used to update the cache. This allows refreshing a particular project
+     * attribute in the cache.
      * @returns the value of the queried project attribute. Can be either a string or a
      * list of strings, depending on the attribute itself (e.g: 'Main' attribute is
      * specified as a list of strings while 'Target' as a string)
      */
-    public async getProjectAttributeValue(
+    public getProjectAttributeValue(
         attribute: string,
         pkg: string = '',
         index: string = '',
+        useCache = true,
     ): Promise<string | string[]> {
-        const params: ExecuteCommandParams = {
-            command: 'als-get-project-attribute-value',
-            arguments: [
-                {
-                    attribute: attribute,
-                    pkg: pkg,
-                    index: index,
-                },
-            ],
+        const queryArgs = {
+            attribute: attribute,
+            pkg: pkg,
+            index: index,
         };
 
-        const attrValue = (await this.adaClient.sendRequest(ExecuteCommandRequest.type, params)) as
-            | string
-            | string[];
-        return attrValue;
+        /**
+         * In the JS Map class, keys are compared using something equivalent to
+         * ===. So two distinct objects with the same deep content will
+         * constitute different keys. A common way of using objects as Map keys
+         * is to stringify them.
+         */
+        const mapKey = JSON.stringify(queryArgs);
+        const cachedPromise = useCache ? this.projectAttributeCache.get(mapKey) : undefined;
+
+        if (cachedPromise === undefined) {
+            const params: ExecuteCommandParams = {
+                command: 'als-get-project-attribute-value',
+                arguments: [queryArgs],
+            };
+
+            const queryPromise = this.adaClient
+                .sendRequest(ExecuteCommandRequest.type, params)
+                .then((value) => {
+                    /**
+                     * Only cache the promise if it was fulfilled.
+                     */
+                    this.projectAttributeCache.set(mapKey, queryPromise);
+                    return value as string | string[];
+                });
+
+            return queryPromise;
+        } else {
+            return cachedPromise;
+        }
     }
 
     /**
@@ -505,6 +677,15 @@ export class ExtensionState {
     public getAdaTaskProvider() {
         return this.adaTaskProvider;
     }
+
+    /**
+     *
+     * @returns path under the project object dir where the VS Code extension can
+     * store temporary state
+     */
+    public async getVSCodeObjectSubdir(): Promise<string> {
+        return path.join(await this.getObjectDir(), 'vscode-ada');
+    }
 }
 
 /**
