@@ -10,7 +10,7 @@ import { AdaConfig, getOrAskForProgram, initializeConfig } from './debugConfigPr
 import { adaExtState, logger, mainOutputChannel } from './extension';
 import { findAdaMain, getProjectFileRelPath, getSymbols } from './helpers';
 import {
-    DEFAULT_PROBLEM_MATCHER,
+    DEFAULT_PROBLEM_MATCHERS,
     SimpleTaskDef,
     TASK_PROVE_FILE_PLAIN_NAME,
     TASK_PROVE_LINE_PLAIN_NAME,
@@ -86,7 +86,36 @@ export const CMD_SPARK_LIMIT_SUBP_ARG = 'ada.spark.limitSubpArg';
 export const CMD_SPARK_LIMIT_REGION_ARG = 'ada.spark.limitRegionArg';
 export const CMD_SPARK_PROVE_SUBP = 'ada.spark.proveSubprogram';
 
+/**
+ * Identifier for the command that shows the extension's output in the Output panel.
+ */
+export const CMD_SHOW_EXTENSION_LOGS = 'ada.showExtensionOutput';
+
+/**
+ * Identifier for the command that shows the output of the ALS for Ada in the Output panel.
+ */
+export const CMD_SHOW_ADA_LS_OUTPUT = 'ada.showAdaLSOutput';
+
+/**
+ * Identifier for the command that shows the output of the ALS for GPR in the Output panel.
+ */
+export const CMD_SHOW_GPR_LS_OUTPUT = 'ada.showGprLSOutput';
+
+/**
+ * Identifier for the command that reloads the currently loaded project on server-side.
+ */
+export const CMD_RELOAD_PROJECT = 'als-reload-project';
+
+/**
+ * Identifier for the command that restarts all the language servers spawned by the extension
+ * (Ada and GPR).
+ */
+export const CMD_RESTART_LANG_SERVERS = 'ada.restartLanguageServers';
+
 export function registerCommands(context: vscode.ExtensionContext, clients: ExtensionState) {
+    context.subscriptions.push(
+        vscode.commands.registerCommand(CMD_RESTART_LANG_SERVERS, restartLanguageServers),
+    );
     context.subscriptions.push(
         vscode.commands.registerCommand('ada.createHelloWorldProject', createHelloWorldProject),
     );
@@ -113,15 +142,15 @@ export function registerCommands(context: vscode.ExtensionContext, clients: Exte
         vscode.commands.registerCommand('ada.subprogramBox', addSubprogramBoxCommand),
     );
     context.subscriptions.push(
-        vscode.commands.registerCommand('ada.showExtensionOutput', () => mainOutputChannel.show()),
+        vscode.commands.registerCommand(CMD_SHOW_EXTENSION_LOGS, () => mainOutputChannel.show()),
     );
     context.subscriptions.push(
-        vscode.commands.registerCommand('ada.showAdaLSOutput', () =>
+        vscode.commands.registerCommand(CMD_SHOW_ADA_LS_OUTPUT, () =>
             clients.adaClient.outputChannel.show(),
         ),
     );
     context.subscriptions.push(
-        vscode.commands.registerCommand('ada.showGprLSOutput', () =>
+        vscode.commands.registerCommand(CMD_SHOW_GPR_LS_OUTPUT, () =>
             clients.gprClient.outputChannel.show(),
         ),
     );
@@ -467,6 +496,24 @@ async function buildAndRunMainAsk() {
     }
 }
 
+/**
+ * Handler for commands that restart language servers launched by the extension.
+ *
+ */
+async function restartLanguageServers() {
+    logger.info('Restarting language servers, updating the process environment first...');
+
+    // Restart the language server clients
+    for (const client of [adaExtState.adaClient, adaExtState.gprClient]) {
+        await client.restart();
+    }
+
+    //  Clear the extension's cache and tasks after restarting the servers
+    adaExtState.clearCacheAndTasks(
+        'Language servers have been restarted, clearing cache and tasks',
+    );
+}
+
 /**
  * Handler for commands that create new files.
  * This function creates a new editor for the given language, focus it,
@@ -679,6 +726,7 @@ async function buildAndDebugSpecifiedMain(
     const adaMain = await findAdaMain(main.fsPath);
     const target = await adaExtState.getTargetPrefix();
     const debugServerAddress = await adaExtState.getDebugServerAddress(useGNATemulator);
+
     if (adaMain) {
         /**
          * The vscode API doesn't provide a way to list both automatically
@@ -715,6 +763,17 @@ async function buildAndDebugSpecifiedMain(
         if (matchingConfig) {
             logger.debug('Found matching config: ' + JSON.stringify(matchingConfig, null, 2));
 
+            // No debug server address specified in matching debug configuration: fallback
+            // to 'localhost:1234' to match GNATemulator's default debug port.
+            if (!matchingConfig.miDebuggerServerAddress && useGNATemulator) {
+                const fallbackDebugServerAddress = 'localhost:1234';
+                logger.debug(
+                    `No debug server address specified in debug configuration for GNATemulator:
+ use '${fallbackDebugServerAddress}' as a fallback`,
+                );
+                matchingConfig.miDebuggerServerAddress = 'localhost:1234';
+            }
+
             // Trying to debug via GNATemulator: run the main via GNATemulator in debug mode
             // before starting the debug session
             if (useGNATemulator) {
@@ -979,7 +1038,7 @@ async function sparkProveSubprogram(
          * empty problem matcher list which would hide problems from the
          * Problems view.
          */
-        task.problemMatchers.length > 0 ? task.problemMatchers : [DEFAULT_PROBLEM_MATCHER],
+        task.problemMatchers.length > 0 ? task.problemMatchers : DEFAULT_PROBLEM_MATCHERS,
     );
 
     /**
