@@ -26,7 +26,7 @@ import { AdaMain, getAdaMains, showErrorMessageWithOpenLogButton } from './helpe
 
 export const TASK_TYPE_ADA = 'ada';
 export const TASK_TYPE_SPARK = 'spark';
-export const DEFAULT_PROBLEM_MATCHER = '$ada';
+export const DEFAULT_PROBLEM_MATCHERS = ['$ada-error', '$ada-warning', '$ada-info'];
 
 /**
  * A type representing task definitions as they appear in tasks.json files.
@@ -73,7 +73,7 @@ const TASK_BUILD_PROJECT: PredefinedTask = {
         command: 'gprbuild',
         args: ['${command:ada.gprProjectArgs}', "'-cargs:ada'", '-gnatef'],
     },
-    problemMatchers: DEFAULT_PROBLEM_MATCHER,
+    problemMatchers: DEFAULT_PROBLEM_MATCHERS,
     taskGroup: vscode.TaskGroup.Build,
 };
 // eslint-disable-next-line max-len
@@ -86,7 +86,7 @@ const TASK_CLEAN_PROJECT = {
         command: 'gprclean',
         args: ['${command:ada.gprProjectArgs}'],
     },
-    problemMatchers: '',
+    problemMatchers: [],
     taskGroup: vscode.TaskGroup.Clean,
 };
 
@@ -103,7 +103,7 @@ export const TASK_GNATCOV_SETUP: PredefinedTask = {
         command: 'gnatcov',
         args: ['setup'],
     },
-    problemMatchers: DEFAULT_PROBLEM_MATCHER,
+    problemMatchers: DEFAULT_PROBLEM_MATCHERS,
 };
 
 const gnatCovTasks: PredefinedTask[] = [TASK_GNATCOV_SETUP];
@@ -136,7 +136,7 @@ const predefinedTasks: PredefinedTask[] = [
                 '-gnatef',
             ],
         },
-        problemMatchers: DEFAULT_PROBLEM_MATCHER,
+        problemMatchers: DEFAULT_PROBLEM_MATCHERS,
     },
     {
         label: 'Compile current file',
@@ -154,7 +154,7 @@ const predefinedTasks: PredefinedTask[] = [
                 '-gnatef',
             ],
         },
-        problemMatchers: DEFAULT_PROBLEM_MATCHER,
+        problemMatchers: DEFAULT_PROBLEM_MATCHERS,
     },
     {
         label: 'Analyze the project with GNAT SAS',
@@ -168,7 +168,7 @@ const predefinedTasks: PredefinedTask[] = [
          * with a problem matcher. Results should be viewed with the
          * `gnatsas report` task below
          */
-        problemMatchers: '',
+        problemMatchers: [],
     },
     {
         label: 'Analyze the current file with GNAT SAS',
@@ -184,7 +184,7 @@ const predefinedTasks: PredefinedTask[] = [
          * the file URIs in the SARIF file are relative to the workspace root
          * folder and SARIF-viewer extensions can correctly resolve them.
          */
-        problemMatchers: '',
+        problemMatchers: [],
     },
     {
         label: 'Create a report after a GNAT SAS analysis',
@@ -205,7 +205,7 @@ const predefinedTasks: PredefinedTask[] = [
          * Analysis results are not printed on stdio so no need to parse them
          * with a problem matcher.
          */
-        problemMatchers: '',
+        problemMatchers: [],
     },
     {
         label: 'Analyze the project with GNAT SAS and produce a report',
@@ -220,7 +220,7 @@ const predefinedTasks: PredefinedTask[] = [
          * Analysis results are not printed on stdio so no need to parse them
          * with a problem matcher.
          */
-        problemMatchers: '',
+        problemMatchers: [],
     },
     {
         label: 'Analyze the current file with GNAT SAS and produce a report',
@@ -235,7 +235,7 @@ const predefinedTasks: PredefinedTask[] = [
          * Analysis results are not printed on stdio so no need to parse them
          * with a problem matcher.
          */
-        problemMatchers: '',
+        problemMatchers: [],
     },
     {
         label: 'Generate documentation from the project',
@@ -244,7 +244,7 @@ const predefinedTasks: PredefinedTask[] = [
             command: 'gnatdoc',
             args: ['${command:ada.gprProjectArgs}'],
         },
-        problemMatchers: '',
+        problemMatchers: [],
     },
     {
         label: 'Create or update GNATtest test framework',
@@ -253,7 +253,7 @@ const predefinedTasks: PredefinedTask[] = [
             command: 'gnattest',
             args: ['${command:ada.gprProjectArgs}'],
         },
-        problemMatchers: '',
+        problemMatchers: [],
     },
     /**
      * SPARK
@@ -265,7 +265,7 @@ const predefinedTasks: PredefinedTask[] = [
             command: 'gnatprove',
             args: ['${command:ada.gprProjectArgs}', '--clean'],
         },
-        problemMatchers: '',
+        problemMatchers: [],
     },
     {
         label: 'Examine project',
@@ -274,7 +274,7 @@ const predefinedTasks: PredefinedTask[] = [
             command: 'gnatprove',
             args: ['${command:ada.gprProjectArgs}', '-j0', '--mode=flow', '-cargs', '-gnatef'],
         },
-        problemMatchers: DEFAULT_PROBLEM_MATCHER,
+        problemMatchers: DEFAULT_PROBLEM_MATCHERS,
     },
     {
         label: 'Examine file',
@@ -291,7 +291,7 @@ const predefinedTasks: PredefinedTask[] = [
                 '-gnatef',
             ],
         },
-        problemMatchers: DEFAULT_PROBLEM_MATCHER,
+        problemMatchers: DEFAULT_PROBLEM_MATCHERS,
     },
     {
         label: 'Examine subprogram',
@@ -307,7 +307,7 @@ const predefinedTasks: PredefinedTask[] = [
                 '-gnatef',
             ],
         },
-        problemMatchers: DEFAULT_PROBLEM_MATCHER,
+        problemMatchers: DEFAULT_PROBLEM_MATCHERS,
     },
     {
         label: 'Prove project',
@@ -316,7 +316,7 @@ const predefinedTasks: PredefinedTask[] = [
             command: 'gnatprove',
             args: ['${command:ada.gprProjectArgs}', '-j0', '-cargs', '-gnatef'],
         },
-        problemMatchers: DEFAULT_PROBLEM_MATCHER,
+        problemMatchers: DEFAULT_PROBLEM_MATCHERS,
     },
     {
         label: TASK_PROVE_FILE_PLAIN_NAME,
@@ -332,7 +332,7 @@ const predefinedTasks: PredefinedTask[] = [
                 '-gnatef',
             ],
         },
-        problemMatchers: DEFAULT_PROBLEM_MATCHER,
+        problemMatchers: DEFAULT_PROBLEM_MATCHERS,
     },
     {
         label: TASK_PROVE_SUPB_PLAIN_NAME,
@@ -347,7 +347,7 @@ const predefinedTasks: PredefinedTask[] = [
                 '-gnatef',
             ],
         },
-        problemMatchers: DEFAULT_PROBLEM_MATCHER,
+        problemMatchers: DEFAULT_PROBLEM_MATCHERS,
     },
     {
         label: TASK_PROVE_REGION_PLAIN_NAME,
@@ -364,7 +364,7 @@ const predefinedTasks: PredefinedTask[] = [
                 '-gnatef',
             ],
         },
-        problemMatchers: DEFAULT_PROBLEM_MATCHER,
+        problemMatchers: DEFAULT_PROBLEM_MATCHERS,
     },
     {
         label: TASK_PROVE_LINE_PLAIN_NAME,
@@ -381,7 +381,7 @@ const predefinedTasks: PredefinedTask[] = [
                 '-gnatef',
             ],
         },
-        problemMatchers: DEFAULT_PROBLEM_MATCHER,
+        problemMatchers: DEFAULT_PROBLEM_MATCHERS,
     },
 ].concat(gnatCovTasks);
 
@@ -404,8 +404,14 @@ export class SimpleTaskProvider implements vscode.TaskProvider {
         }
 
         const result: vscode.Task[] = [];
-        const targetPrefix = await adaExtState.getTargetPrefix();
-        const isNativeProject = await adaExtState.isNativeProject();
+        const targetPrefix = await adaExtState.getTargetPrefix().catch((err) => {
+            logger.error('Error in task provider:\n' + err);
+            return '';
+        });
+        const isNativeProject = await adaExtState.isNativeProject().catch((err) => {
+            logger.error('Error in task provider:\n' + err);
+            return true;
+        });
 
         /**
          * Start with the list of predefined tasks.
@@ -417,7 +423,12 @@ export class SimpleTaskProvider implements vscode.TaskProvider {
              * Add tasks based on the Mains of the project.
              */
             taskDeclsToOffer.push(
-                ...(await getAdaMains()).flatMap((main) => {
+                ...(
+                    await getAdaMains().catch((err) => {
+                        logger.error('Error in task provider:\n' + err);
+                        return [];
+                    })
+                ).flatMap((main) => {
                     if (token?.isCancellationRequested) {
                         throw new vscode.CancellationError();
                     }
@@ -434,7 +445,7 @@ export class SimpleTaskProvider implements vscode.TaskProvider {
                                 '-gnatef',
                             ],
                         },
-                        problemMatchers: DEFAULT_PROBLEM_MATCHER,
+                        problemMatchers: DEFAULT_PROBLEM_MATCHERS,
                         taskGroup: vscode.TaskGroup.Build,
                     };
 
@@ -468,7 +479,7 @@ export class SimpleTaskProvider implements vscode.TaskProvider {
                             type: this.taskType,
                             compound: [buildTask.label, runTask.label],
                         },
-                        problemMatchers: '',
+                        problemMatchers: [],
                     };
 
                     const tasks = [buildTask, runTask, buildAndRunTask];
@@ -498,7 +509,7 @@ export class SimpleTaskProvider implements vscode.TaskProvider {
                             // This task is run before some debug tasks.
                             // Problem is, it's a watch script, and since it never exits, VSCode
                             // complains. All this is needed so VSCode just lets it run.
-                            problemMatchers: '',
+                            problemMatchers: [],
                         };
 
                         const buildAndRunGNATemulatorTask: PredefinedTask = {
@@ -507,7 +518,7 @@ export class SimpleTaskProvider implements vscode.TaskProvider {
                                 type: this.taskType,
                                 compound: [buildTask.label, runGNATemulatorTask.label],
                             },
-                            problemMatchers: '',
+                            problemMatchers: [],
                         };
 
                         const buildAndRunGNATemulatorTaskForDebug: PredefinedTask = {
@@ -516,7 +527,7 @@ export class SimpleTaskProvider implements vscode.TaskProvider {
                                 type: this.taskType,
                                 compound: [buildTask.label, runGNATemulatorTaskForDebug.label],
                             },
-                            problemMatchers: '',
+                            problemMatchers: [],
                         };
 
                         tasks.push(
@@ -554,7 +565,7 @@ export class SimpleTaskProvider implements vscode.TaskProvider {
                         command: 'gprbuild',
                         args: ['-P', harnessPrj, "'-cargs:ada'", '-gnatef'],
                     },
-                    problemMatchers: DEFAULT_PROBLEM_MATCHER,
+                    problemMatchers: DEFAULT_PROBLEM_MATCHERS,
                     taskGroup: vscode.TaskGroup.Build,
                 });
             }
